@@ -355,7 +355,7 @@ for causal, headdim, bs_seqlen, nheads in itertools.product(
             q=q_fwd,
             k=k_fwd,
             v=v_fwd,
-            is_inference=is_infer,
+            generate_stats=not is_infer,
             attn_scale=attn_scale,
             use_causal_mask=is_causal,
             dropout=dropout_tuple if is_dropout else None,
@@ -562,7 +562,7 @@ for causal, headdim, bs_seqlen, nheads in itertools.product(
             descale_s=descale_s_fwd,
             scale_s=scale_s_fwd,
             scale_o=scale_o_fwd,
-            is_inference=is_infer,
+            generate_stats=not is_infer,
             attn_scale=attn_scale,
             use_causal_mask=is_causal,
             use_padding_mask=False,
