@@ -730,45 +730,6 @@ class Conv_dgrad_attributes : public Attributes<Conv_dgrad_attributes> {
     }
 };
 
-class Matmul_attributes : public Attributes<Matmul_attributes> {
-    friend class Attributes<Matmul_attributes>;
-    friend class MatmulNode;
-    friend class INode;
-
-    double padding_value = 0.0;
-
-   public:
-    enum class input_names { A, B, M_override, N_override, K_override };
-    std::unordered_map<input_names, std::shared_ptr<Tensor_attributes>> inputs;
-    enum class output_names { C };
-    std::unordered_map<output_names, std::shared_ptr<Tensor_attributes>> outputs;
-    NLOHMANN_DEFINE_TYPE_INTRUSIVE(Matmul_attributes, name, compute_data_type, inputs, outputs, padding_value)
-
-    Matmul_attributes&
-    set_m_override(std::shared_ptr<Tensor_attributes> const& value) {
-        inputs[input_names::M_override] = value;
-        return *this;
-    }
-
-    Matmul_attributes&
-    set_n_override(std::shared_ptr<Tensor_attributes> const& value) {
-        inputs[input_names::N_override] = value;
-        return *this;
-    }
-
-    Matmul_attributes&
-    set_k_override(std::shared_ptr<Tensor_attributes> const& value) {
-        inputs[input_names::K_override] = value;
-        return *this;
-    }
-
-    Matmul_attributes&
-    set_padding(double const padding_val) {
-        padding_value = padding_val;
-        return *this;
-    }
-};
-
 class Matmul_fp8_attributes : public Attributes<Matmul_fp8_attributes> {
     friend class Attributes<Matmul_fp8_attributes>;
     friend class MatmulFP8Node;
@@ -806,6 +767,70 @@ class Matmul_fp8_attributes : public Attributes<Matmul_fp8_attributes> {
         padding_value = padding_val;
         return *this;
     }
+
+    double
+    get_padding() const {
+        return padding_value;
+    }
+};
+
+class Matmul_attributes : public Attributes<Matmul_attributes> {
+    friend class Attributes<Matmul_attributes>;
+    friend class MatmulNode;
+    friend class INode;
+
+    double padding_value = 0.0;
+
+   public:
+    enum class input_names { A, B, M_override, N_override, K_override };
+    std::unordered_map<input_names, std::shared_ptr<Tensor_attributes>> inputs;
+    enum class output_names { C };
+    std::unordered_map<output_names, std::shared_ptr<Tensor_attributes>> outputs;
+    NLOHMANN_DEFINE_TYPE_INTRUSIVE(Matmul_attributes, name, compute_data_type, inputs, outputs, padding_value)
+
+    Matmul_attributes&
+    clone_fp8_attributes(Matmul_fp8_attributes const& attributes) {
+        auto m_override = attributes.inputs.find(Matmul_fp8_attributes::input_names::M_override);
+        if (m_override != attributes.inputs.end()) {
+            set_m_override(m_override->second);
+        }
+        auto n_override = attributes.inputs.find(Matmul_fp8_attributes::input_names::N_override);
+        if (n_override != attributes.inputs.end()) {
+            set_n_override(n_override->second);
+        }
+        auto k_override = attributes.inputs.find(Matmul_fp8_attributes::input_names::K_override);
+        if (k_override != attributes.inputs.end()) {
+            set_k_override(k_override->second);
+        }
+
+        set_padding(attributes.get_padding());
+
+        return *this;
+    }
+
+    Matmul_attributes&
+    set_m_override(std::shared_ptr<Tensor_attributes> const& value) {
+        inputs[input_names::M_override] = value;
+        return *this;
+    }
+
+    Matmul_attributes&
+    set_n_override(std::shared_ptr<Tensor_attributes> const& value) {
+        inputs[input_names::N_override] = value;
+        return *this;
+    }
+
+    Matmul_attributes&
+    set_k_override(std::shared_ptr<Tensor_attributes> const& value) {
+        inputs[input_names::K_override] = value;
+        return *this;
+    }
+
+    Matmul_attributes&
+    set_padding(double const padding_val) {
+        padding_value = padding_val;
+        return *this;
+    }
 };
 
 class Pointwise_attributes : public Attributes<Pointwise_attributes> {
@@ -822,6 +847,10 @@ class Pointwise_attributes : public Attributes<Pointwise_attributes> {
     std::optional<float> relu_upper_clip;
     std::optional<float> relu_lower_clip_slope;
 
+    std::optional<float> swish_beta;
+    std::optional<float> elu_alpha;
+    std::optional<float> softplus_beta;
+
    public:
     enum class input_names { IN_0, IN_1, IN_2 };
     std::unordered_map<input_names, std::shared_ptr<Tensor_attributes>> inputs;
@@ -836,7 +865,10 @@ class Pointwise_attributes : public Attributes<Pointwise_attributes> {
                                    axis,
                                    relu_lower_clip,
                                    relu_upper_clip,
-                                   relu_lower_clip_slope)
+                                   relu_lower_clip_slope,
+                                   swish_beta,
+                                   elu_alpha,
+                                   softplus_beta)
 
     Pointwise_attributes&
     set_mode(PointwiseMode_t const value) {
@@ -872,6 +904,24 @@ class Pointwise_attributes : public Attributes<Pointwise_attributes> {
         this->relu_upper_clip = value;
         return *this;
     }
+
+    Pointwise_attributes&
+    set_swish_beta(float const value) {
+        this->swish_beta = value;
+        return *this;
+    }
+
+    Pointwise_attributes&
+    set_elu_alpha(float const value) {
+        this->elu_alpha = value;
+        return *this;
+    }
+
+    Pointwise_attributes&
+    set_softplus_beta(float const value) {
+        this->softplus_beta = value;
+        return *this;
+    }
 };
 
 class Instancenorm_backward_attributes : public Attributes<Instancenorm_backward_attributes> {
@@ -1074,13 +1124,20 @@ class Reduction_attributes : public Attributes<Reduction_attributes> {
     friend class INode;
 
     std::optional<ReductionMode_t> mode;
+    bool is_deterministic = false;
 
    public:
     enum class input_names { X };
     std::unordered_map<input_names, std::shared_ptr<Tensor_attributes>> inputs;
     enum class output_names { Y };
     std::unordered_map<output_names, std::shared_ptr<Tensor_attributes>> outputs;
-    NLOHMANN_DEFINE_TYPE_INTRUSIVE(Reduction_attributes, name, compute_data_type, inputs, outputs, mode)
+    NLOHMANN_DEFINE_TYPE_INTRUSIVE(Reduction_attributes,
+                                   name,
+                                   compute_data_type,
+                                   inputs,
+                                   outputs,
+                                   mode,
+                                   is_deterministic)
 
     std::optional<ReductionMode_t>
     get_mode() const {
@@ -1092,6 +1149,17 @@ class Reduction_attributes : public Attributes<Reduction_attributes> {
         mode = value;
         return *this;
     }
+
+    bool
+    get_is_deterministic() const {
+        return is_deterministic;
+    }
+
+    Reduction_attributes&
+    set_is_deterministic(bool value) {
+        is_deterministic = value;
+        return *this;
+    }
 };
 
 class Rng_attributes : public Attributes<Rng_attributes> {
@@ -1181,7 +1249,7 @@ class Resample_attributes : public Attributes<Resample_attributes> {
     friend class ResampleNode;
     friend class INode;
 
-    std::optional<bool> is_inference;
+    std::optional<bool> generate_index;
     ResampleMode_t resample_mode;
     PaddingMode_t padding_mode;
     std::vector<cudnnFraction_t> pre_padding;
@@ -1200,7 +1268,7 @@ class Resample_attributes : public Attributes<Resample_attributes> {
                                    name,
                                    inputs,
                                    outputs,
-                                   is_inference,
+                                   generate_index,
                                    resample_mode,
                                    padding_mode,
                                    pre_padding,
@@ -1285,10 +1353,16 @@ class Resample_attributes : public Attributes<Resample_attributes> {
     }
 
     auto
-    set_is_inference(bool const value) -> Resample_attributes& {
-        is_inference = value;
+    set_generate_index(bool const value) -> Resample_attributes& {
+        generate_index = value;
         return *this;
     }
+
+    [[deprecated]]
+    auto
+    set_is_inference(bool const value) -> Resample_attributes& {
+        return set_generate_index(!value);
+    }
 };
 
 class Reshape_attributes : public Attributes<Reshape_attributes> {
@@ -1499,10 +1573,17 @@ class Rmsnorm_backward_attributes : public Attributes<Rmsnorm_backward_attribute
 //         return *this;
 //     }
 // };
+template <typename DerivedClass>
+class SDPANodeBase;
+class CompositeSDPANode;
+class UnifiedSDPANode;
 
 class SDPA_attributes : public Attributes<SDPA_attributes> {
     friend class Attributes<SDPA_attributes>;
-    friend class SDPANode;
+    friend class SDPANodeBase<CompositeSDPANode>;
+    friend class CompositeSDPANode;
+    friend class SDPANodeBase<UnifiedSDPANode>;
+    friend class UnifiedSDPANode;
     friend class Graph;
 
     using Tensor_t = std::shared_ptr<Tensor_attributes>;
@@ -1511,7 +1592,7 @@ class SDPA_attributes : public Attributes<SDPA_attributes> {
     using AttentionScoreModifier_t =
         std::function<Tensor_t(std::shared_ptr<Graph>, std::shared_ptr<Tensor_attributes>)>;
 
-    std::optional<bool> is_inference;
+    std::optional<bool> generate_stats;
     bool alibi_mask   = false;
     bool padding_mask = false;
     std::optional<int64_t> left_bound;
@@ -1521,6 +1602,13 @@ class SDPA_attributes : public Attributes<SDPA_attributes> {
     std::optional<float> attn_scale_value;
     std::optional<int> max_seq_len_kv;
     AttentionScoreModifier_t attention_score_modifier = nullptr;
+    DataType_t mma_core_mode                          = DataType_t::NOT_SET;
+
+    // Deprecated fields for backward compatibility with SDPA_fp8_attributes
+    bool causal_mask              = false;
+    bool causal_mask_bottom_right = false;
+
+    AttentionImplementation_t implementation = AttentionImplementation_t::AUTO;
 
     bool
     has_causal_like_masking() const {
@@ -1546,31 +1634,73 @@ class SDPA_attributes : public Attributes<SDPA_attributes> {
         Dropout_mask,
         Dropout_scale,
         Page_table_K,
-        Page_table_V
+        Page_table_V,
+        // FP8-specific scaling inputs
+        Descale_Q,
+        Descale_K,
+        Descale_V,
+        Descale_S,
+        Scale_S,
+        Scale_O,
+        SINK_TOKEN,
     };
     std::unordered_map<input_names, std::shared_ptr<Tensor_attributes>> inputs;
-    enum class output_names { O, Stats, RNG_DUMP };
+    enum class output_names { O, Stats, Max, Sum_exp, RNG_DUMP, Amax_S, Amax_O };
     std::unordered_map<output_names, std::shared_ptr<Tensor_attributes>> outputs;
+    // Convenience struct for named access to SDPA outputs
+    struct SDPA_outputs {
+        std::shared_ptr<Tensor_attributes> O;         ///< Main attention output tensor
+        std::shared_ptr<Tensor_attributes> Stats;     ///< Statistics/softmax output (when generate_stats=true)
+        std::shared_ptr<Tensor_attributes> Max;       ///< Max output tensor
+        std::shared_ptr<Tensor_attributes> Sum_exp;   ///< Sum_exp output tensor
+        std::shared_ptr<Tensor_attributes> RNG_DUMP;  ///< Random number generator dump for dropout
+                                                      ///< check why we don't return RNG_DUMP this way
+        std::shared_ptr<Tensor_attributes> Amax_S;    ///< FP8 absolute maximum for attention scores
+        std::shared_ptr<Tensor_attributes> Amax_O;    ///< FP8 absolute maximum for output tensor
+    };
+
     NLOHMANN_DEFINE_TYPE_INTRUSIVE(SDPA_attributes,
                                    name,
                                    inputs,
                                    outputs,
-                                   is_inference,
+                                   generate_stats,
                                    alibi_mask,
                                    padding_mask,
                                    dropout_probability,
                                    attn_scale_value,
                                    max_seq_len_kv,
+                                   mma_core_mode,
                                    left_bound,
                                    right_bound,
-                                   diagonal_alignment)
+                                   diagonal_alignment,
+                                   causal_mask,
+                                   causal_mask_bottom_right,
+                                   implementation)
 
     SDPA_attributes&
-    set_is_inference(bool const value) {
-        is_inference = value;
+    set_generate_stats(bool const value) {
+        generate_stats = value;
         return *this;
     }
 
+    SDPA_attributes&
+    set_logit_max(std::shared_ptr<Tensor_attributes> value) {
+        outputs[SDPA_attributes::output_names::Max] = std::move(value);
+        return *this;
+    }
+
+    SDPA_attributes&
+    set_score_sum_exp(std::shared_ptr<Tensor_attributes> value) {
+        outputs[SDPA_attributes::output_names::Sum_exp] = std::move(value);
+        return *this;
+    }
+
+    [[deprecated]]
+    SDPA_attributes&
+    set_is_inference(bool const value) {
+        return set_generate_stats(!value);
+    }
+
     SDPA_attributes&
     set_attn_scale(std::shared_ptr<Tensor_attributes> value) {
         inputs[SDPA_attributes::input_names::Attn_scale] = std::move(value);
@@ -1601,6 +1731,13 @@ class SDPA_attributes : public Attributes<SDPA_attributes> {
         return *this;
     }
 
+    // Internal function - do not use directly in application code
+    SDPA_attributes&
+    _set_mma_core_mode(DataType_t const value) {
+        mma_core_mode = value;
+        return *this;
+    }
+
     SDPA_attributes&
     set_seq_len_q(std::shared_ptr<Tensor_attributes> value) {
         inputs[SDPA_attributes::input_names::SEQ_LEN_Q] = std::move(value);
@@ -1630,7 +1767,7 @@ class SDPA_attributes : public Attributes<SDPA_attributes> {
                 set_diagonal_band_right_bound(0);
             }
         }
-
+        causal_mask = value;
         return *this;
     }
 
@@ -1645,6 +1782,7 @@ class SDPA_attributes : public Attributes<SDPA_attributes> {
                 set_diagonal_band_right_bound(0);
             }
         }
+        causal_mask_bottom_right = value;
         return *this;
     }
 
@@ -1714,132 +1852,54 @@ class SDPA_attributes : public Attributes<SDPA_attributes> {
         max_seq_len_kv = value;
         return *this;
     }
+
+    SDPA_attributes&
+    set_sink_token(std::shared_ptr<Tensor_attributes> value) {
+        inputs[SDPA_attributes::input_names::SINK_TOKEN] = std::move(value);
+        return *this;
+    }
+
+    SDPA_attributes&
+    set_implementation(AttentionImplementation_t value) {
+        implementation = value;
+        return *this;
+    }
+
+    // Implementation is in sdpa_support_surface.h
+    error_t
+    validate_sdpa_support_surface(const detail::Context& context, int64_t s_kv, bool is_paged_k, bool is_paged_v) const;
+
+    // Internal function - do not use directly in application code
+    void
+    _auto_select_implementation(const detail::Context& context) {
+        if (verify_sdpa_support_surface_for_implementation(context, AttentionImplementation_t::UNIFIED).is_good()) {
+            implementation = AttentionImplementation_t::UNIFIED;
+            CUDNN_FE_LOG_LABEL_ENDL("INFO: Auto-selected SDPA implementation UNIFIED");
+        } else if (verify_sdpa_support_surface_for_implementation(context, AttentionImplementation_t::COMPOSITE)
+                       .is_good()) {
+            implementation = AttentionImplementation_t::COMPOSITE;
+            CUDNN_FE_LOG_LABEL_ENDL("INFO: Auto-selected SDPA implementation COMPOSITE");
+        } else {
+            // Leave `implementation` with its previous value (usually AUTO).
+            CUDNN_FE_LOG_LABEL_ENDL("ERROR: No suitable SDPA implementation for given SDPA_attributes");
+        }
+    }
+
+   private:
+    // Check whether implementation `impl` supports the requested features. `impl` must not be AUTO.
+    // (The `implementation` member variable is ignored.)
+    error_t
+    verify_sdpa_support_surface_for_implementation(const detail::Context& context,
+                                                   AttentionImplementation_t impl) const;
 };
 
-class SDPA_fp8_attributes : public Attributes<SDPA_fp8_attributes> {
-    friend class Attributes<SDPA_fp8_attributes>;
-    friend class SDPAFP8Node;
-    friend class Graph;
-
-    std::optional<bool> is_inference;
-    bool padding_mask             = false;
-    bool causal_mask              = false;
-    bool causal_mask_bottom_right = false;
-    std::optional<float> dropout_probability;
-    std::optional<float> attn_scale_value;
-
-   public:
-    enum class input_names {
-        Q,
-        K,
-        V,
-        Attn_scale,
-        Bias,
-        SEQ_LEN_Q,
-        SEQ_LEN_KV,
-        Seed,
-        Offset,
-        Dropout_mask,
-        Dropout_scale,
-
-        Descale_Q,
-        Descale_K,
-        Descale_V,
-        Descale_S,
-        Scale_S,
-        Scale_O,
-    };
-    std::unordered_map<input_names, std::shared_ptr<Tensor_attributes>> inputs;
-
-    enum class output_names { O, Stats, Amax_S, Amax_O };
-    std::unordered_map<output_names, std::shared_ptr<Tensor_attributes>> outputs;
-
-    NLOHMANN_DEFINE_TYPE_INTRUSIVE(SDPA_fp8_attributes,
-                                   name,
-                                   inputs,
-                                   outputs,
-                                   is_inference,
-                                   padding_mask,
-                                   causal_mask,
-                                   causal_mask_bottom_right,
-                                   dropout_probability,
-                                   attn_scale_value)
-
-    SDPA_fp8_attributes&
-    set_is_inference(bool const value) {
-        is_inference = value;
-        return *this;
-    }
-
-    SDPA_fp8_attributes&
-    set_attn_scale(std::shared_ptr<Tensor_attributes> value) {
-        inputs[SDPA_fp8_attributes::input_names::Attn_scale] = value;
-        return *this;
-    }
-
-    SDPA_fp8_attributes&
-    set_attn_scale(float const value) {
-        attn_scale_value = value;
-        return *this;
-    }
-
-    SDPA_fp8_attributes&
-    set_bias(std::shared_ptr<Tensor_attributes> value) {
-        inputs[SDPA_fp8_attributes::input_names::Bias] = value;
-        return *this;
-    }
-
-    SDPA_fp8_attributes&
-    set_padding_mask(bool const value) {
-        padding_mask = value;
-        return *this;
-    }
-
-    SDPA_fp8_attributes&
-    set_seq_len_q(std::shared_ptr<Tensor_attributes> value) {
-        inputs[SDPA_fp8_attributes::input_names::SEQ_LEN_Q] = value;
-        return *this;
-    }
-
-    SDPA_fp8_attributes&
-    set_seq_len_kv(std::shared_ptr<Tensor_attributes> value) {
-        inputs[SDPA_fp8_attributes::input_names::SEQ_LEN_KV] = value;
-        return *this;
-    }
-
-    SDPA_fp8_attributes&
-    set_causal_mask(bool const value) {
-        causal_mask = value;
-        return *this;
-    }
-
-    SDPA_fp8_attributes&
-    set_dropout(float const probability,
-                std::shared_ptr<Tensor_attributes> seed,
-                std::shared_ptr<Tensor_attributes> offset) {
-        dropout_probability                              = probability;
-        inputs[SDPA_fp8_attributes::input_names::Seed]   = seed;
-        inputs[SDPA_fp8_attributes::input_names::Offset] = offset;
-        return *this;
-    }
-
-    SDPA_fp8_attributes&
-    set_dropout(std::shared_ptr<Tensor_attributes> mask, std::shared_ptr<Tensor_attributes> scale) {
-        inputs[SDPA_fp8_attributes::input_names::Dropout_mask]  = mask;
-        inputs[SDPA_fp8_attributes::input_names::Dropout_scale] = scale;
-        return *this;
-    }
-
-    SDPA_fp8_attributes&
-    set_causal_mask_bottom_right(bool const value) {
-        causal_mask_bottom_right = value;
-        return *this;
-    }
-};
+// Type alias for backward compatibility - SDPA_fp8_attributes is now an alias to SDPA_attributes
+// All FP8 functionality is unified in SDPA_attributes with the mma_core_mode field
+using SDPA_fp8_attributes = SDPA_attributes;
 
 class SDPA_backward_attributes : public Attributes<SDPA_backward_attributes> {
     friend class Attributes<SDPA_backward_attributes>;
-    friend class SDPABackwardNode;
+    friend class CompositeSDPABackwardNode;
     friend class Graph;
     using Tensor_t = std::shared_ptr<Tensor_attributes>;
     using Graph_t  = std::shared_ptr<Graph>;
@@ -1890,9 +1950,10 @@ class SDPA_backward_attributes : public Attributes<SDPA_backward_attributes> {
         Dropout_mask,
         Dropout_scale,
         Dropout_scale_inv,
+        SINK_TOKEN,
     };
     std::unordered_map<input_names, std::shared_ptr<Tensor_attributes>> inputs;
-    enum class output_names { dQ, dK, dV, dBias, RNG_DUMP };
+    enum class output_names { dQ, dK, dV, dBias, RNG_DUMP, DSINK_TOKEN };
     std::unordered_map<output_names, std::shared_ptr<Tensor_attributes>> outputs;
     NLOHMANN_DEFINE_TYPE_INTRUSIVE(SDPA_backward_attributes,
                                    name,
@@ -2066,6 +2127,18 @@ class SDPA_backward_attributes : public Attributes<SDPA_backward_attributes> {
         is_deterministic_algorithm = value;
         return *this;
     }
+
+    SDPA_backward_attributes&
+    set_sink_token(std::shared_ptr<Tensor_attributes> value) {
+        inputs[SDPA_backward_attributes::input_names::SINK_TOKEN] = value;
+        return *this;
+    }
+
+    SDPA_backward_attributes&
+    set_dsink_token(std::shared_ptr<Tensor_attributes> value) {
+        outputs[SDPA_backward_attributes::output_names::DSINK_TOKEN] = value;
+        return *this;
+    }
 };
 
 class SDPA_fp8_backward_attributes : public Attributes<SDPA_fp8_backward_attributes> {
@@ -2204,25 +2277,16 @@ class Softmax_attributes : public Attributes<Softmax_attributes> {
     friend class SoftmaxNode;
     friend class INode;
 
-    std::optional<bool> use_stats;
-    std::optional<bool> use_M_Zinv;
-
    public:
-    enum class input_names { P };
+    enum class input_names { P, SINK };
     std::unordered_map<input_names, std::shared_ptr<Tensor_attributes>> inputs;
-    enum class output_names { S, Stats, M, Zinv };
+    enum class output_names { S, Stats, Max, Sum_exp };
     std::unordered_map<output_names, std::shared_ptr<Tensor_attributes>> outputs;
-    NLOHMANN_DEFINE_TYPE_INTRUSIVE(Softmax_attributes, name, compute_data_type, inputs, outputs, use_stats, use_M_Zinv)
+    NLOHMANN_DEFINE_TYPE_INTRUSIVE(Softmax_attributes, name, compute_data_type, inputs, outputs)
 
     Softmax_attributes&
-    has_stats(bool const value) {
-        use_stats = value;
-        return *this;
-    }
-
-    Softmax_attributes&
-    has_M_Zinv(bool const value) {
-        use_M_Zinv = value;
+    set_sink(std::shared_ptr<Tensor_attributes> value) {
+        inputs[Softmax_attributes::input_names::SINK] = value;
         return *this;
     }
 };
@@ -2416,7 +2480,8 @@ class Block_scale_dequantize_attributes : public Attributes<Block_scale_dequanti
     friend class BlockScaleDequantizeNode;
     friend class Graph;
 
-    std::optional<int32_t> block_size;
+    std::vector<int32_t> block_size;
+    bool is_negative_scale;
 
    public:
     enum class input_names { X, scale };
@@ -2428,11 +2493,47 @@ class Block_scale_dequantize_attributes : public Attributes<Block_scale_dequanti
                                    compute_data_type,
                                    inputs,
                                    outputs,
-                                   block_size)
+                                   block_size,
+                                   is_negative_scale)
 
     Block_scale_dequantize_attributes&
-    set_block_size(int32_t const value) {
-        block_size = value;
+    set_block_size(int32_t const value, int32_t idx = 0) {
+        if (idx < 0) {
+            return *this;
+        }
+        if (static_cast<int32_t>(block_size.size()) < idx + 1) {
+            block_size.resize(idx + 1, 1);
+        }
+        block_size[idx] = value;
+        return *this;
+    }
+
+    Block_scale_dequantize_attributes&
+    set_block_size(const int32_t* values, int32_t len = 1) {
+        if (len < 1) {
+            return *this;
+        }
+        if (static_cast<int32_t>(block_size.size()) < len) {
+            block_size.resize(len);
+        }
+        std::copy(values, values + len, block_size.begin());
+        return *this;
+    }
+
+    Block_scale_dequantize_attributes&
+    set_block_size(const std::vector<int32_t>& values) {
+        block_size = values;
+        return *this;
+    }
+
+    bool
+    get_is_negative_scale() const {
+        return is_negative_scale;
+    }
+
+    Block_scale_dequantize_attributes&
+    set_is_negative_scale(bool value) {
+        is_negative_scale = value;
         return *this;
     }
 };
