@@ -129,13 +129,15 @@ class INode {
         RESHAPE,
         RMSNORM,
         RNG,
-        SCALED_DOT_PRODUCT_ATTENTION,
         SLICE,
         WGRAD,
         PAGED_CACHE_LOAD,
         BLOCK_SCALE_QUANTIZE,
         BLOCK_SCALE_DEQUANTIZE,
-        CONCATENATE
+        CONCATENATE,
+        ADALAYERNORM,
+        DADALAYERNORM,
+        UNIFIED_SDPA,
     };
     Type tag;
 
@@ -159,7 +161,9 @@ class INode {
     softmax(std::shared_ptr<Tensor_attributes> p,
             Softmax_attributes attributes,
             std::shared_ptr<Tensor_attributes> s,
-            std::shared_ptr<Tensor_attributes> stats);
+            std::shared_ptr<Tensor_attributes> stats,
+            std::shared_ptr<Tensor_attributes> max,
+            std::shared_ptr<Tensor_attributes> sum_exp);
 
     void
     softmax(std::shared_ptr<Tensor_attributes> p,
