@@ -29,6 +29,7 @@
 #include <iomanip>
 #include <sstream>
 #include <cmath>
+#include <cstdint>
 
 #include <cuda_fp16.h>
 #include <cuda_bf16.h>
@@ -672,6 +673,9 @@ enum class DataType_t {
     FAST_FLOAT_FOR_FP8,
     FP8_E8M0,
     FP4_E2M1,
+    INT4,
+    COMPLEX_FP32,
+    COMPLEX_FP64,
 };
 
 NLOHMANN_JSON_SERIALIZE_ENUM(DataType_t,
@@ -694,6 +698,10 @@ NLOHMANN_JSON_SERIALIZE_ENUM(DataType_t,
                                  {DataType_t::FAST_FLOAT_FOR_FP8, "FAST_FLOAT_FOR_FP8"},
                                  {DataType_t::FP8_E8M0, "FP8_E8M0"},
                                  {DataType_t::FP4_E2M1, "FP4_E2M1"},
+                                 {DataType_t::INT4, "INT4"},
+                                 {DataType_t::COMPLEX_FP32, "COMPLEX_FP32"},
+                                 {DataType_t::COMPLEX_FP64, "COMPLEX_FP64"},
+
                              })
 
 enum class ReductionMode_t {
@@ -936,6 +944,14 @@ NLOHMANN_JSON_SERIALIZE_ENUM(DiagonalAlignment_t,
                                  {DiagonalAlignment_t::BOTTOM_RIGHT, "BOTTOM_RIGHT"},
                              })
 
+enum class AttentionImplementation_t { AUTO, COMPOSITE, UNIFIED };
+NLOHMANN_JSON_SERIALIZE_ENUM(AttentionImplementation_t,
+                             {
+                                 {AttentionImplementation_t::AUTO, "AUTO"},
+                                 {AttentionImplementation_t::COMPOSITE, "COMPOSITE"},
+                                 {AttentionImplementation_t::UNIFIED, "UNIFIED"},
+                             })
+
 namespace detail {
 
 inline size_t
@@ -972,7 +988,7 @@ get_data_type_size(DataType_t const data_type) {
 }
 
 inline std::vector<float>
-get_abili_slope(int64_t const n_heads) {
+get_alibi_slope(int64_t const n_heads) {
     std::vector<float> slope;
 #ifdef _MSC_VER
 #pragma warning(push)
@@ -1063,7 +1079,7 @@ convert_to_cudnn_type(cudnn_frontend::DataType_t const mode, cudnnDataType_t& cu
             return cudnnStatus_t::CUDNN_STATUS_INVALID_VALUE;
 #endif
         case DataType_t::FP8_E8M0:
-#if (CUDNN_VERSION >= 90700)  // TODO: v9.99 is new feature branch; switch to release branch when ready
+#if (CUDNN_VERSION >= 90700)
             NV_CUDNN_FE_DYNAMIC_CHECK_CUDNN_BACKEND_VERSION(90700, cudnnStatus_t::CUDNN_STATUS_INVALID_VALUE);
             cudnn_mode = CUDNN_DATA_FP8_E8M0;
             return cudnnStatus_t::CUDNN_STATUS_SUCCESS;
@@ -1071,14 +1087,37 @@ convert_to_cudnn_type(cudnn_frontend::DataType_t const mode, cudnnDataType_t& cu
             return cudnnStatus_t::CUDNN_STATUS_INVALID_VALUE;
 #endif
         case DataType_t::FP4_E2M1:
-#if (CUDNN_VERSION >= 90700)  // TODO: v9.99 is new feature branch; switch to release branch when ready
+#if (CUDNN_VERSION >= 90700)
             NV_CUDNN_FE_DYNAMIC_CHECK_CUDNN_BACKEND_VERSION(90700, cudnnStatus_t::CUDNN_STATUS_INVALID_VALUE);
             cudnn_mode = CUDNN_DATA_FP4_E2M1;
             return cudnnStatus_t::CUDNN_STATUS_SUCCESS;
 #else
             return cudnnStatus_t::CUDNN_STATUS_INVALID_VALUE;
 #endif
-
+        case DataType_t::INT4:
+#if (CUDNN_VERSION >= 91100)
+            NV_CUDNN_FE_DYNAMIC_CHECK_CUDNN_BACKEND_VERSION(91000, cudnnStatus_t::CUDNN_STATUS_INVALID_VALUE);
+            cudnn_mode = CUDNN_DATA_INT4;
+            return cudnnStatus_t::CUDNN_STATUS_SUCCESS;
+#else
+            return cudnnStatus_t::CUDNN_STATUS_INVALID_VALUE;
+#endif
+        case DataType_t::COMPLEX_FP32:
+#if (CUDNN_VERSION >= 91400)
+            NV_CUDNN_FE_DYNAMIC_CHECK_CUDNN_BACKEND_VERSION(91400, cudnnStatus_t::CUDNN_STATUS_INVALID_VALUE);
+            cudnn_mode = CUDNN_DATA_COMPLEX_FP32;
+            return cudnnStatus_t::CUDNN_STATUS_SUCCESS;
+#else
+            return cudnnStatus_t::CUDNN_STATUS_INVALID_VALUE;
+#endif
+        case DataType_t::COMPLEX_FP64:
+#if (CUDNN_VERSION >= 91400)
+            NV_CUDNN_FE_DYNAMIC_CHECK_CUDNN_BACKEND_VERSION(91400, cudnnStatus_t::CUDNN_STATUS_INVALID_VALUE);
+            cudnn_mode = CUDNN_DATA_COMPLEX_FP64;
+            return cudnnStatus_t::CUDNN_STATUS_SUCCESS;
+#else
+            return cudnnStatus_t::CUDNN_STATUS_INVALID_VALUE;
+#endif
 #ifndef NO_DEFAULT_IN_SWITCH
         default:
             return cudnnStatus_t::CUDNN_STATUS_INVALID_VALUE;
@@ -2149,14 +2188,24 @@ convert_from_cudnn_type(cudnnDataType_t const cudnn_mode) {
         case CUDNN_DATA_FAST_FLOAT_FOR_FP8:
             return DataType_t::FAST_FLOAT_FOR_FP8;
 #endif
-#if (CUDNN_VERSION >= 90700)  // TODO: v9.99 is new feature branch; switch to release branch when ready
+#if (CUDNN_VERSION >= 90700)
         case CUDNN_DATA_FP8_E8M0:
             return DataType_t::FP8_E8M0;
 #endif
-#if (CUDNN_VERSION >= 90700)  // TODO: v9.99 is new feature branch; switch to release branch when ready
+#if (CUDNN_VERSION >= 90700)
         case CUDNN_DATA_FP4_E2M1:
             return DataType_t::FP4_E2M1;
 #endif
+#if (CUDNN_VERSION >= 91100)
+        case CUDNN_DATA_INT4:
+            return DataType_t::INT4;
+#endif
+#if (CUDNN_VERSION >= 91400)
+        case CUDNN_DATA_COMPLEX_FP32:
+            return DataType_t::COMPLEX_FP32;
+        case CUDNN_DATA_COMPLEX_FP64:
+            return DataType_t::COMPLEX_FP64;
+#endif
 #ifndef NO_DEFAULT_IN_SWITCH
         default:
             return DataType_t::NOT_SET;
@@ -2165,6 +2214,64 @@ convert_from_cudnn_type(cudnnDataType_t const cudnn_mode) {
     return DataType_t::NOT_SET;
 }
 
+static size_t
+get_element_size_in_bits(cudnn_frontend::DataType_t datatype) {
+    switch (datatype) {
+        case DataType_t::INT8x32:
+            return 256;
+            break;
+#if (CUDNN_VERSION >= 91400)
+        case DataType_t::COMPLEX_FP64:
+            return 128;
+            break;
+#endif
+        case DataType_t::DOUBLE:
+        case DataType_t::INT64:
+#if (CUDNN_VERSION >= 91400)
+        case DataType_t::COMPLEX_FP32:
+#endif
+            return 64;
+            break;
+        case DataType_t::FLOAT:
+        case DataType_t::INT32:
+        case DataType_t::INT8x4:
+        case DataType_t::UINT8x4:
+            return 32;
+            break;
+        case DataType_t::HALF:
+        case DataType_t::BFLOAT16:
+            return 16;
+            break;
+        case DataType_t::INT8:
+        case DataType_t::UINT8:
+#if (CUDNN_VERSION >= 8600)
+        case DataType_t::FP8_E4M3:
+        case DataType_t::FP8_E5M2:
+#endif
+#if (CUDNN_VERSION >= 8700)
+        case DataType_t::FAST_FLOAT_FOR_FP8:
+#endif
+#if (CUDNN_VERSION >= 90700)
+        case DataType_t::FP8_E8M0:
+#endif
+            return 8;
+            break;
+#if (CUDNN_VERSION >= 90700)
+        case DataType_t::FP4_E2M1:
+#if (CUDNN_VERSION >= 91100)
+        case DataType_t::INT4:
+#endif
+            return 4;
+#endif
+        case DataType_t::BOOLEAN:
+            return 1;
+            break;
+        default:
+            return 0;
+            break;
+    }
+}
+
 // To be deprecated. Only exists as setReductionOp(cudnnReduceTensorOp_t mode) requires it.
 static inline cudnn_frontend::ReductionMode_t
 convert_from_cudnn_type(cudnnReduceTensorOp_t const cudnn_mode) {
