@@ -84,7 +84,9 @@ PyGraph::tensor(std::vector<int64_t> const& dim,
                 bool const& is_virtual,
                 bool const& is_pass_by_value,
                 std::shared_ptr<cudnn_frontend::graph::Tensor_attributes> const& ragged_offset,
-                std::string const& name) {
+                cudnn_frontend::TensorReordering_t const reordering_type,
+                std::string const& name,
+                int64_t const& uid) {
     auto props = cudnn_frontend::graph::Tensor_attributes()
                      .set_data_type(data_type)
                      .set_is_virtual(is_virtual)
@@ -92,8 +94,13 @@ PyGraph::tensor(std::vector<int64_t> const& dim,
                      .set_dim(dim)
                      .set_stride(stride)
                      .set_ragged_offset(ragged_offset)
+                     .set_reordering_type(reordering_type)
                      .set_name(name);
 
+    if (uid != -1) {
+        props.set_uid(uid);
+    }
+
     return graph->tensor(props);
 }
 
@@ -262,6 +269,25 @@ PyGraph::conv_wgrad(std::shared_ptr<cudnn_frontend::graph::Tensor_attributes>& i
     return DW;
 }
 
+std::shared_ptr<cudnn_frontend::graph::Tensor_attributes>
+PyGraph::block_scale_dequantize(std::shared_ptr<cudnn_frontend::graph::Tensor_attributes>& input,
+                                std::shared_ptr<cudnn_frontend::graph::Tensor_attributes>& descale,
+                                std::vector<int32_t> const& block_size,
+                                bool const is_negative_scale,
+                                cudnn_frontend::DataType_t const& compute_data_type,
+                                std::string const& name) {
+    auto attributes = cudnn_frontend::graph::Block_scale_dequantize_attributes()
+                          .set_block_size(block_size)
+                          .set_is_negative_scale(is_negative_scale)
+                          .set_compute_data_type(compute_data_type)
+                          .set_name(name);
+    if (compute_data_type != cudnn_frontend::DataType_t::NOT_SET) {
+        attributes.set_compute_data_type(compute_data_type);
+    }
+    auto output = graph->block_scale_dequantize(input, descale, attributes);
+    return output;
+}
+
 std::shared_ptr<cudnn_frontend::graph::Tensor_attributes>
 PyGraph::matmul(std::shared_ptr<cudnn_frontend::graph::Tensor_attributes>& A,
                 std::shared_ptr<cudnn_frontend::graph::Tensor_attributes>& B,
@@ -374,13 +400,13 @@ PyGraph::get_knobs_for_engine(int64_t const engine_id) {
 void
 PyGraph::build_plans(BuildPlanPolicy_t const policy) {
     // TODO: Add multithreaded support in python
-    auto status = graph->build_plans(handle, policy, false);
+    auto status = graph->build_plans(policy, false);
     throw_if(status.is_bad(), status.get_code(), status.get_message());
 }
 
 void
 PyGraph::build_plan_at_index(int64_t const index) {
-    auto status = graph->build_plan_at_index(handle, index);
+    auto status = graph->build_plan_at_index(index);
     throw_if(status.is_bad(), status.get_code(), status.get_message());
 }
 
@@ -401,7 +427,7 @@ PyGraph::build() {
 
 void
 PyGraph::check_support() {
-    auto status = graph->check_support(handle);
+    auto status = graph->check_support();
     throw_if(status.is_bad(), status.get_code(), status.get_message());
 }
 
@@ -434,7 +460,7 @@ PyGraph::serialize() const {
 }
 
 void
-PyGraph::deserialize(py::object const& pyobj) {
+PyGraph::deserialize(std::optional<std::intptr_t> handle_, py::object const& pyobj) {
     if (py::isinstance<py::str>(pyobj)) {
         json j = json::parse(pyobj.cast<std::string>());
 
@@ -443,6 +469,10 @@ PyGraph::deserialize(py::object const& pyobj) {
         throw_if(status.is_bad(), status.get_code(), status.get_message());
 
     } else {
+        // If handle is provided, use it (AoT compilation).
+        cudnnHandle_t handle =
+            handle_.has_value() ? static_cast<cudnnHandle_t>((void*)(handle_.value())) : this->handle;
+
         std::vector<uint8_t> data = pyobj.cast<std::vector<uint8_t>>();
         auto status               = graph->deserialize(handle, data);
 
@@ -450,6 +480,12 @@ PyGraph::deserialize(py::object const& pyobj) {
     }
 }
 
+void
+PyGraph::deserialize(py::object const& pyobj) {
+    // Call the overloaded version with default handle (nullopt)
+    deserialize(std::nullopt, pyobj);
+}
+
 void
 PyGraph::update_cuda_graph(std::intptr_t handle,
                            std::unordered_map<cudnn_frontend::graph::Tensor_attributes::uid_t, std::intptr_t> var_pack,
@@ -563,7 +599,8 @@ init_pygraph_submodule(py::module_& m) {
                       std::optional<std::intptr_t>,
                       py::object,
                       py::object,
-                      std::shared_ptr<KernelCache>>(),
+                      std::shared_ptr<KernelCache>,
+                      std::shared_ptr<cudnn_frontend::DeviceProperties>>(),
              py::arg_v("name", "test_graph"),
              py::arg_v("io_data_type", cudnn_frontend::DataType_t::NOT_SET),
              py::arg_v("intermediate_data_type", cudnn_frontend::DataType_t::NOT_SET),
@@ -571,7 +608,8 @@ init_pygraph_submodule(py::module_& m) {
              py::arg_v("handle", std::nullopt),
              py::arg_v("sm_count", py::none()),
              py::arg_v("sm_version", py::none()),
-             py::arg_v("kernel_cache", nullptr))
+             py::arg_v("kernel_cache", nullptr),
+             py::arg_v("device_property", nullptr))
         .def("tensor_like",
              py::overload_cast<std::shared_ptr<cudnn_frontend::graph::Tensor_attributes> const&, std::string const&>(
                  &PyGraph::tensor_like),
@@ -586,7 +624,9 @@ init_pygraph_submodule(py::module_& m) {
              py::arg_v{"is_virtual", false},
              py::arg_v{"is_pass_by_value", false},
              py::arg_v{"ragged_offset", nullptr},
-             py::arg_v("name", ""))
+             py::arg_v{"reordering_type", cudnn_frontend::TensorReordering_t::NONE},
+             py::arg_v("name", ""),
+             py::arg_v("uid", -1))
         .def("genstats",
              &PyGraph::genstats,
              py::arg("input"),
@@ -798,6 +838,28 @@ init_pygraph_submodule(py::module_& m) {
                 Returns:
                     cudnn_tensor: The result of reduction operation.
             )pbdoc")
+        .def("block_scale_dequantize",
+             &PyGraph::block_scale_dequantize,
+             py::arg("input"),
+             py::arg("descale"),
+             py::arg("block_size"),
+             py::arg_v("is_negative_scale", false),
+             py::arg_v("compute_data_type", cudnn_frontend::DataType_t::NOT_SET),
+             py::arg_v("name", ""),
+             R"pbdoc(
+                Dequantize an input tensor to other dimensions without changing the actual memory layout.
+                
+                Args:
+                    input (cudnn_tensor): The input tensor to dequantize.
+                    descale (cudnn_tensor): The scale tensor for dequantization.
+                    block_size (List[int]): The block size for dequantization.
+                    is_negative_scale (Optional[bool]): Whether the scale values can be negative. Default is False.
+                    compute_data_type (Optional[cudnn.data_type]): The data type for computation. Default is NOT_SET.
+                    name (Optional[str]): A name for the operation.
+                
+                Returns:
+                    cudnn_tensor: The dequantized output tensor.
+            )pbdoc")
         .def("reshape",
              &PyGraph::reshape,
              py::arg("input"),
@@ -891,7 +953,11 @@ init_pygraph_submodule(py::module_& m) {
         .def("populate_cuda_graph", &PyGraph::populate_cuda_graph)
         .def("update_cuda_graph", &PyGraph::update_cuda_graph)
         .def("serialize", &PyGraph::serialize)
-        .def("deserialize", &PyGraph::deserialize)
+        .def("deserialize",
+             (void(PyGraph::*)(std::optional<std::intptr_t>, py::object const&)) & PyGraph::deserialize,
+             py::arg("handle_"),
+             py::arg("pyobj"))
+        .def("deserialize", (void(PyGraph::*)(py::object const&)) & PyGraph::deserialize, py::arg("pyobj"))
         .def("_execute_plan_at_index", &PyGraph::execute_plan_at_index)
         .def("__repr__", [](PyGraph const& pygraph) {
             std::stringstream ss;
