@@ -135,10 +135,10 @@ TEST_CASE("Graph key", "[graph][key]") {
     REQUIRE(graph.create_execution_plans({fe::HeurMode_t::A}).is_good());
     REQUIRE(key == graph.key());
 
-    REQUIRE(graph.check_support(handle).is_good());
+    REQUIRE(graph.check_support().is_good());
     REQUIRE(key == graph.key());
 
-    REQUIRE(graph.build_plans(handle).is_good());
+    REQUIRE(graph.build_plans().is_good());
     REQUIRE(key == graph.key());
 
     cudnnDestroy(handle);
@@ -215,10 +215,10 @@ TEST_CASE("Graph key dynamic shape", "[graph][key][dynamic_shape]") {
         REQUIRE(graph.create_execution_plans({fe::HeurMode_t::A}).is_good());
         REQUIRE(key == graph.key());
 
-        REQUIRE(graph.check_support(handle).is_good());
+        REQUIRE(graph.check_support().is_good());
         REQUIRE(key == graph.key());
 
-        REQUIRE(graph.build_plans(handle).is_good());
+        REQUIRE(graph.build_plans().is_good());
         REQUIRE(key == graph.key());
 
         cudnnDestroy(handle);
@@ -446,7 +446,7 @@ TEST_CASE("sdpa graph serialization", "[graph][serialize]") {
 
     auto sdpa_options = fe::graph::SDPA_attributes()
                             .set_name("flash_attention")
-                            .set_is_inference(false)
+                            .set_generate_stats(true)
                             .set_attn_scale(attn_scale)
                             .set_alibi_mask(true)
                             .set_diagonal_band_right_bound(0)
@@ -535,8 +535,7 @@ TEST_CASE("sdpa backward graph serialization", "[graph][serialize]") {
                                      .set_diagonal_band_right_bound(0)
                                      .set_attn_scale(attn_scale)
                                      .set_bias(bias)
-                                     .set_dropout(0.1f, dropout_seed, dropout_offset)
-                                     .set_deterministic_algorithm(true);
+                                     .set_dropout(0.1f, dropout_seed, dropout_offset);
 
     auto [dQ, dK, dV] = graph.sdpa_backward(q, k, v, o, dO, stats, sdpa_backward_options);
 
