@@ -117,7 +117,7 @@ def decode_mask(
     return out
 
 
-def causal_mask(sdpa_graph, q_kt_tensor):
+def causal_mask(sdpa_graph, q_kt_tensor, neg_inf):
 
     row_index = sdpa_graph.gen_index(input=q_kt_tensor, axis=2)
     row_index.set_data_type(cudnn.data_type.INT32)
@@ -130,7 +130,7 @@ def causal_mask(sdpa_graph, q_kt_tensor):
     )
     mask.set_data_type(cudnn.data_type.BOOLEAN)
 
-    out = sdpa_graph.binary_select(input0=q_kt_tensor, input1=q_kt_tensor, mask=mask)
+    out = sdpa_graph.binary_select(input0=q_kt_tensor, input1=neg_inf, mask=mask)
 
     return out
 
@@ -302,7 +302,7 @@ def test_sdpa_with_flexible_graph(cudnn_handle):
         v=container_v,
         seq_len_q=seq_len_q,
         seq_len_kv=seq_len_kv,
-        is_inference=True,
+        generate_stats=False,
         attn_scale=attn_scale,
         use_causal_mask=False,
         paged_attention_k_table=page_table_k,  # Page Table K: Tensor containing offsets to the container with K blocks
@@ -480,7 +480,7 @@ def test_sdpa_with_arrow_mask(cudnn_handle):
         q=q,
         k=k,
         v=v,
-        is_inference=True,
+        generate_stats=False,
         attn_scale=attn_scale,
         score_mod=partial(
             arrow_mask,
@@ -583,7 +583,7 @@ def test_sdpa_with_document_mask(cudnn_handle):
         q=q,
         k=k,
         v=v,
-        is_inference=True,
+        generate_stats=False,
         attn_scale=attn_scale,
         score_mod=partial(
             document_mask,
